@@ -1,4 +1,4 @@
-# Copyright 2005-6 by Roger Bivand
+# Copyright 2005-7 by Roger Bivand
 spautolm <- function(formula, data = list(), listw, weights=NULL,
     na.action=na.fail, verbose=FALSE, tol.opt=.Machine$double.eps^(2/3),
     family="SAR", method="full", interval=c(-1,0.999), zero.policy=FALSE,
@@ -119,6 +119,47 @@ spautolm <- function(formula, data = list(), listw, weights=NULL,
         LL0 <- .opt.fit.SparseM(lambda=as.numeric(0), Y=Y, X=X, n=n, W=W,
             W_J=W_J, I=I, weights=Sweights, sum_lw=sum_lw, family=family,
             verbose=verbose, cholAlloc=cholAlloc, tol.solve=tol.solve)
+#        weights <- diag(Sweights)
+    }  else if (method == "Matrix") {
+        if (listw$style %in% c("W", "S") && !can.sim)
+        stop("Matrix method requires symmetric weights")
+        if (listw$style %in% c("B", "C", "U") && 
+ 	    !(is.symmetric.glist(listw$neighbours, listw$weights)))
+	    stop("Matrix method requires symmetric weights")
+        I <- as_dgCMatrix_I(n)
+	I <- as(I, "CsparseMatrix")
+	W <- as_dgRMatrix_listw(listw)
+	W <- as(W, "CsparseMatrix")
+        if (family == "CAR") if (!isTRUE(all.equal(W, t(W))))
+	    warning("Non-symmetric spatial weights in CAR model")
+# Jacobian only from symmetric W_J, W can be asymmetric though
+        if (listw$style %in% c("W", "S") & can.sim) {
+	    W_J <- as_dsTMatrix_listw(listw2U(similar.listw(listw)))
+#	    similar <- TRUE
+	} else W_J <- as_dsTMatrix_listw(listw)
+	gc(FALSE)
+        Sweights <- as(Diagonal(x=weights), "sparseMatrix")
+# do line search
+        opt <- optimize(.opt.fit.Matrix, lower=interval[1],
+            upper=interval[2], maximum=TRUE,
+            tol = tol.opt, Y=Y, X=X, n=n, W=W, W_J=W_J, I=I,
+            weights=Sweights, sum_lw=sum_lw, family=family,
+            verbose=verbose, tol.solve=tol.solve)
+        lambda <- opt$maximum
+        names(lambda) <- "lambda"
+        LL <- opt$objective
+# get GLS coefficients
+        fit <- .SPAR.fit(lambda=lambda, Y=Y, X=X, n=n, W=W, I=I,
+            weights=Sweights, family=family, out=TRUE, tol.solve=tol.solve)
+# create residuals and fitted values (Cressie 1993, p. 564)
+	fit$signal_trend <- drop(X %*% fit$coefficients)
+	fit$signal_stochastic <- drop(lambda * W %*% (Y - fit$signal_trend))
+	fit$fitted.values <- fit$signal_trend + fit$signal_stochastic
+	fit$residuals <- drop(Y - fit$fitted.values)
+# get null LL
+        LL0 <- .opt.fit.Matrix(lambda=as.numeric(0), Y=Y, X=X, n=n, W=W,
+            W_J=W_J, I=I, weights=Sweights, sum_lw=sum_lw, family=family,
+            verbose=verbose, tol.solve=tol.solve)
 #        weights <- diag(Sweights)
     } else stop("unknown method")
     res <- list(fit=fit, lambda=lambda, LL=LL, LL0=LL0, call=match.call(),
@@ -158,7 +199,8 @@ spautolm <- function(formula, data = list(), listw, weights=NULL,
     SSE <- .SPAR.fit(lambda=lambda, Y=Y, X=X, n=n, W=W, weights=weights,
         I=I, family=family, out=FALSE, tol.solve=tol.solve)
     s2 <- SSE/n
-    Jacobian <- log(det(chol((I - lambda * W_J), nsubmax=cholAlloc$nsubmax, 
+    Det <- get("det", "package:SparseM")
+    Jacobian <- log(Det(chol((I - lambda * W_J), nsubmax=cholAlloc$nsubmax, 
 	nnzlmax=cholAlloc$nnzlmax, tmpmax=cholAlloc$tmpmax))^2)
     gc(FALSE)
     ret <- ((1/ifelse((length(grep("CAR", family)) != 0), 2, 1))*Jacobian +
@@ -167,6 +209,25 @@ spautolm <- function(formula, data = list(), listw, weights=NULL,
     ret
 }
 
+.opt.fit.Matrix <- function(lambda, Y, X, n, W, W_J, I, weights, sum_lw,
+    family="SAR_x", verbose=TRUE, tol.solve=.Machine$double.eps) {
+# fitting function called from optimize()
+    SSE <- .SPAR.fit(lambda=lambda, Y=Y, X=X, n=n, W=W, weights=weights,
+        I=I, family=family, out=FALSE, tol.solve=tol.solve)
+    s2 <- SSE/n
+    CHOL <- try(chol(as((I - lambda * W_J), "dsCMatrix")), silent=TRUE)
+    if (class(CHOL) == "try-error") {
+        Jacobian <- NA
+    } else {
+        Jacobian <- sum(2*log(diag(CHOL)))
+    }
+    gc(FALSE)
+    ret <- ((1/ifelse((length(grep("CAR", family)) != 0), 2, 1))*Jacobian +
+	(1/2)*sum_lw - ((n/2)*log(2*pi)) - (n/2)*log(s2) - (1/(2*(s2)))*SSE)
+    if (verbose)  cat("lambda:", lambda, "function", ret, "Jacobian", Jacobian, "SSE", SSE, "\n")
+    ret
+}
+
 .SPAR.fit <- function(lambda, Y, X, n, W, weights, I, family,
     out=FALSE, tol.solve=.Machine$double.eps) {
     dmmf <- eval(parse(text=family))
